#version 300 es
precision highp float;

// these are our textures
uniform sampler2D msdf;

uniform float fade;
uniform float visibilityRadius;
uniform float far_z;
uniform float near_z;
uniform float width;
uniform float height;
uniform vec2 unitRange;
uniform vec4 bgColor;

// this comes from the vertex shader
in vec2 uv_a;
in vec2 uv_b;
in vec4 fontColor;
in float msdf_mix;

out vec4 outputColor;

float median(float r, float g, float b) {
    return max(min(r, g), min(max(r, g), b));
}

float screenPxRange(vec2 uv) {
    //ivec2 ts = textureSize(msdf, 0);
    //vec2 unitRange = vec2(pxRange)/vec2(ts);
    vec2 screenTexSize = vec2(1.0)/fwidth(uv);
    return max(0.5*dot(unitRange, screenTexSize), 1.0);
}

float screenPxDistance(vec3 msdf_rgb, vec2 uv) {
    float sd = median(msdf_rgb.r, msdf_rgb.g, msdf_rgb.b);
    return screenPxRange(uv)*(sd - 0.5);
}

//float near = 1.0;
//float far = 4400.0 + 400.0;
//float width = 1920.0 * 2.0;
//float height = 1080.0 * 2.0;

float linearizeDepth(float depth)
{
    return (2.0 * near_z * far_z) / (far_z + near_z - (depth * 2.0 - 1.0) * (far_z - near_z));
}

void main()
{
    vec4 backgroundColor = bgColor;
    vec4 fgColor = fontColor;

    vec3 msdf_rgb_a = texture( msdf, uv_a ).rgb;
    vec3 msdf_rgb_b = texture( msdf, uv_b ).rgb;

    //vec2 sz = vec2(textureSize( msdf, 0 ));
    //float dx_a = dFdx( uv_a.x ) * sz.x;
    //float dy_a = dFdy( uv_a.y ) * sz.y;
    //float dx_b = dFdx( uv_b.x ) * sz.x;
    //float dy_b = dFdy( uv_b.y ) * sz.y;
    //float toPixels_a = 8.0 * inversesqrt( dx_a * dx_a + dy_a * dy_a );
    //float toPixels_b = 8.0 * inversesqrt( dx_b * dx_b + dy_b * dy_b );
    //float sigDist_a = median( msdf_rgb_a.r, msdf_rgb_a.g, msdf_rgb_a.b ) - 0.5;
    //float sigDist_b = median( msdf_rgb_b.r, msdf_rgb_b.g, msdf_rgb_b.b ) - 0.5;
    //float opacity_a = clamp( sigDist_a * toPixels_a + 0.5, 0.0, 1.0 );
    //float opacity_b = clamp( sigDist_b * toPixels_b + 0.5, 0.0, 1.0 );
    //float opacity = mix(opacity_a, opacity_b, msdf_mix);

    float screenPxDistance_a = screenPxDistance(msdf_rgb_a, uv_a);
    float screenPxDistance_b = screenPxDistance(msdf_rgb_b, uv_b);
    float screenPxDistance_mix = mix(screenPxDistance_a, screenPxDistance_b, msdf_mix);
    float opacity = clamp(screenPxDistance_mix + 0.5, 0.0, 1.0);

    if (opacity < 0.5)
        discard;

    //outputColor = vec4(vec3(fgColor),1.0); //mix(bgColor, fgColor, opacity);
    float z = linearizeDepth(gl_FragCoord.z) / far_z;
    vec2 screenPos = abs(vec2(
            (gl_FragCoord.x / width) * 2.0 - 1.0,
            (gl_FragCoord.y / height) * 2.0 - 1.0
            ));
    // INFO: this needs to be adjusted if field of view is adjusted
    //float d = smoothstep(0.2, 0.8, z + 0.2 * (screenPos.x + screenPos.y));
    float l = 0.2;
    float d = smoothstep(l, 1.0-l, z + l * (screenPos.x + screenPos.y));
    d = min(d, fade);
    outputColor = vec4(mix(fgColor.rgb, backgroundColor.rgb, d),1.0);
    //outputColor = vec4(mix(1.0 - msdf_rgb_a.rgb, backgroundColor.rgb, d),1.0);

    //outputColor = vec4(opacity, 1.0-opacity, 0.0, 1.0);
    //outputColor = vec4(1.0-msdf_rgb_a.rgb, 1.0);
    //outputColor = vec4(1.0,0.0,1.0,1.0);
}
